@@ -1,9 +1,10 @@
-from __future__ import unicode_literals
 from __future__ import absolute_import
+from __future__ import unicode_literals
 
 import logging
+
+from pypuppetdb.QueryBuilder import (EqualsOperator)
 from pypuppetdb.utils import json_to_datetime
-from pypuppetdb.QueryBuilder import *
 
 log = logging.getLogger(__name__)
 
@@ -52,6 +53,7 @@ class Event(object):
     :ivar item: :obj:`dict` with information about the item/resource this\
         event was triggered for.
     """
+
     def __init__(self, node, status, timestamp, hash_, title, property_,
                  message, new_value, old_value, type_, class_, execution_path,
                  source_file, line_number):
@@ -63,17 +65,11 @@ class Event(object):
             self.failed = False
         self.timestamp = json_to_datetime(timestamp)
         self.hash_ = hash_
-        self.item = {}
-        self.item['title'] = title
-        self.item['type'] = type_
-        self.item['property'] = property_
-        self.item['message'] = message
-        self.item['old'] = old_value
-        self.item['new'] = new_value
-        self.item['class'] = class_
-        self.item['execution_path'] = execution_path
-        self.item['source_file'] = source_file
-        self.item['line_number'] = line_number
+        self.item = {'title': title, 'type': type_, 'property': property_,
+                     'message': message, 'old': old_value,
+                     'new': new_value, 'class': class_,
+                     'execution_path': execution_path, 'source_file': source_file,
+                     'line_number': line_number}
         self.__string = '{0}[{1}]/{2}'.format(self.item['type'],
                                               self.item['title'],
                                               self.hash_)
@@ -84,9 +80,6 @@ class Event(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
 
 class Report(object):
     """This object represents a report. Unless otherwise specified all
@@ -120,6 +113,9 @@ class Report(object):
     :param noop: (Default `False`) A flag indicating weather the report was\
         produced by a noop run.
     :type noop: :obj:`bool`
+    :param noop_pending: (Default `False`) A flag indicating weather the \
+        report had pending changes produced by a noop run.
+    :type noop_pending: :obj:`bool`
     :param metrics: (Optional) All metrics associated with this report.
     :type metrics: :obj:`list` containing :obj:`dict` with Metrics
     :param logs: (Optional) All logs associated with this report.
@@ -135,6 +131,9 @@ class Report(object):
         error. Can be one of 'explicitly_requested', 'on_failure',\
         'not_used' not 'null'.
     :type cached_catalog_status: :obj:`string`
+    :param producer: (Optional) The certname of the Puppet Master that\
+        sent the report to PuppetDB
+    :type producer: :obj:`string`
 
     :ivar node: The hostname this report originated from.
     :ivar hash\_: Unique identifier of this report.
@@ -161,13 +160,16 @@ class Report(object):
     :ivar cached_catalog_status: :obj:`string` identifying if this Puppet run\
         used a cached catalog, if so weather it was a result of an error or\
         otherwise.
+    :ivar producer: :obj:`string` representing the certname of the Puppet\
+        Master that sent the report to PuppetDB
     """
+
     def __init__(self, api, node, hash_, start, end, received, version,
                  format_, agent_version, transaction, status=None,
                  metrics={}, logs={}, environment=None,
-                 noop=False, code_id=None, catalog_uuid=None,
-                 cached_catalog_status=None):
-
+                 noop=False, noop_pending=False, code_id=None,
+                 catalog_uuid=None, cached_catalog_status=None,
+                 producer=None):
         self.node = node
         self.hash_ = hash_
         self.start = json_to_datetime(start)
@@ -179,12 +181,13 @@ class Report(object):
         self.run_time = self.end - self.start
         self.transaction = transaction
         self.environment = environment
-        self.status = 'noop' if noop else status
+        self.status = 'noop' if noop and noop_pending else status
         self.metrics = metrics
         self.logs = logs
         self.code_id = code_id
         self.catalog_uuid = catalog_uuid
         self.cached_catalog_status = cached_catalog_status
+        self.producer = producer
         self.__string = '{0}'.format(self.hash_)
 
         self.__api = api
@@ -195,9 +198,6 @@ class Report(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
     def events(self, **kwargs):
         """Get all events for this report. Additional arguments may also be
         specified that will be passed to the query function.
@@ -226,6 +226,7 @@ class Fact(object):
         fact's value.
     :ivar environment: :obj:`string` holding the fact's environment
     """
+
     def __init__(self, node, name, value, environment=None):
         self.node = node
         self.name = name
@@ -239,9 +240,6 @@ class Fact(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
 
 class Resource(object):
     """This object represents a resource. Unless otherwise specified all
@@ -280,6 +278,7 @@ class Resource(object):
     :ivar environment: :obj:`string` The environment of the node associated\
         with this resource.
     """
+
     def __init__(self, node, name, type_, tags, exported, sourcefile,
                  sourceline, environment=None, parameters={}):
         self.node = node
@@ -300,9 +299,6 @@ class Resource(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
 
 class Node(object):
     """This object represents a node. It additionally has some helper methods
@@ -326,11 +322,20 @@ class Node(object):
             collected.
     :type facts_timestamp: :obj:`string` formatted as\
             ``%Y-%m-%dT%H:%M:%S.%fZ``
-    :param status: (default `None`) Status of the node\
-            changed | unchanged | unreported | failed
+    :param status_report: (default `None`) Status of latest report \
+            from the node changed | unchanged | failed
     :type status: :obj:`string`
+    :param noop: (Default `False`) A flag indicating whether the latest \
+        report of the node was produced by a noop run.
+    :type noop: :obj:`bool`
+    :param noop_pending: (Default `False`) A flag indicating whether \
+        the latest report of the node had pending changes \
+        produced by a noop run.
+    :type noop_pending: :obj:`bool`
     :param events: (default `None`) Counted events from latest Report
     :type events: :obj:`dict`
+    :param unreported: (default `False`) if node is considered unreported
+    :type unreported_time: :obj:`bool`
     :param unreported_time: (default `None`) Time since last report
     :type unreported_time: :obj:`string`
     :param report_environment: (default 'production') The environment of the\
@@ -371,15 +376,17 @@ class Node(object):
     :ivar cached_catalog_status: :obj:`string` the status of the cached\
             catalog from the last puppet run.
     """
+
     def __init__(self, api, name, deactivated=None, expired=None,
                  report_timestamp=None, catalog_timestamp=None,
-                 facts_timestamp=None, status=None, events=None,
-                 unreported_time=None, report_environment='production',
+                 facts_timestamp=None, status_report=None,
+                 noop=False, noop_pending=False, events=None,
+                 unreported=False, unreported_time=None,
+                 report_environment='production',
                  catalog_environment='production',
                  facts_environment='production',
                  latest_report_hash=None, cached_catalog_status=None):
         self.name = name
-        self.status = status
         self.events = events
         self.unreported_time = unreported_time
         self.report_timestamp = report_timestamp
@@ -391,6 +398,13 @@ class Node(object):
         self.latest_report_hash = latest_report_hash
         self.cached_catalog_status = cached_catalog_status
 
+        if unreported:
+            self.status = 'unreported'
+        elif noop and noop_pending:
+            self.status = 'noop'
+        else:
+            self.status = status_report
+
         if deactivated is not None:
             self.deactivated = json_to_datetime(deactivated)
         else:
@@ -421,9 +435,6 @@ class Node(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
     def facts(self, **kwargs):
         """Get all facts of this node. Additional arguments may also be
         specified that will be passed to the query function.
@@ -506,6 +517,9 @@ class Catalog(object):
     :type code_id: :obj:`string`
     :param catalog_uuid: Universally unique identifier of this catalog.
     :type catalog_uuid: :obj:`string`
+    :param producer: The certname of the Puppet Master that sent the catalog\
+        to PuppetDB
+    :type producer: :obj:`string`
 
     :ivar node: :obj:`string` Name of the host
     :ivar version: :obj:`string` Catalog version from Puppet
@@ -521,9 +535,13 @@ class Catalog(object):
     :ivar code_id: :obj:`string` ties the catalog to the Puppet code that\
         generated the catalog
     :ivar catalog_uuid: :obj:`string` uniquely identifying this catalog.
+    :ivar producer: :obj:`string` of the Puppet Master that sent the catalog\
+        to PuppetDB
     """
+
     def __init__(self, node, edges, resources, version, transaction_uuid,
-                 environment=None, code_id=None, catalog_uuid=None):
+                 environment=None, code_id=None, catalog_uuid=None,
+                 producer=None):
 
         self.node = node
         self.version = version
@@ -531,6 +549,7 @@ class Catalog(object):
         self.environment = environment
         self.code_id = code_id
         self.catalog_uuid = catalog_uuid
+        self.producer = producer
 
         self.resources = dict()
         for resource in resources:
@@ -550,10 +569,8 @@ class Catalog(object):
 
         self.edges = []
         for edge in edges:
-            identifier_source = edge['source_type'] + \
-                '[' + edge['source_title'] + ']'
-            identifier_target = edge['target_type'] + \
-                '[' + edge['target_title'] + ']'
+            identifier_source = edge['source_type'] + '[' + edge['source_title'] + ']'
+            identifier_target = edge['target_type'] + '[' + edge['target_title'] + ']'
             e = Edge(source=self.resources[identifier_source],
                      target=self.resources[identifier_target],
                      relationship=edge['relationship'],
@@ -570,18 +587,12 @@ class Catalog(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
-
     def get_resources(self):
-        try:
-            return self.resources.itervalues()
-        except AttributeError:
-            return self.resources.values()
+        return self.resources.values()
 
     def get_resource(self, resource_type, resource_title):
         identifier = resource_type + \
-            '[' + resource_title + ']'
+                     '[' + resource_title + ']'
         return self.resources[identifier]
 
     def get_edges(self):
@@ -607,6 +618,7 @@ class Edge(object):
     :ivar relationship: :obj:`string` Name of the Puppet Resource relationship
     :ivar node: :obj:`string` The name of the node that owns this relationship
     """
+
     def __init__(self, source, target, relationship, node=None):
         self.source = source
         self.target = target
@@ -622,5 +634,45 @@ class Edge(object):
     def __str__(self):
         return str('{0}').format(self.__string)
 
-    def __unicode__(self):
-        return self.__string
+
+class Inventory(object):
+    """This object represents a Node Inventory entry returned from
+    the Inventory endpoint.
+
+    :param node: The certname of the node associated with the inventory.
+    :type node: :obj:`string`
+    :param time: The time at which PuppetDB received the facts in the
+        inventory.
+    :type time: :obj:`string` formatted as ``%Y-%m-%dT%H:%M:%S.%fZ``
+    :param environment: The environment associated with the inventory's
+        certname.
+    :type environment: :obj:`string`
+    :param facts: The dictionary of key-value pairs for the nodes
+        assosciated facts.
+    :type facts: :obj:`dict`
+    :param trusted: The trusted data from the node.
+    :type trusted: :obj:`dict`
+
+    :ivar node: The certname of the node associated with the inventory.
+    :ivar time: The time at which PuppetDB received the facts in the
+        inventory.
+    :ivar environment: The environment associated with the inventory's
+        certname.
+    :ivar facts: The dictionary of key-value pairs for the nodes
+        assosciated facts.
+    :ivar trusted: The trusted data from the node.
+    """
+
+    def __init__(self, node, time, environment, facts, trusted):
+        self.node = node
+        self.time = json_to_datetime(time)
+        self.environment = environment
+        self.facts = facts
+        self.trusted = trusted
+        self.__string = self.node
+
+    def __repr__(self):
+        return str('<Inventory: {0}>').format(self.__string)
+
+    def __str__(self):
+        return str("{0}").format(self.__string)
